<?php
namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use SM\Traits\TQuery;

class Sm00052002DescontoMaximoServicoPerfilAcessoFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getNomePerfilAcessoDescontoMaximoServico() {

        $sql = "SELECT DISTINCT PA.NMPERFILACESSO, QT.NRPERFILACESSO
                    FROM FCT_PERCDESCPERFIL QT
                    INNER JOIN PERFILACESSO PA 
                    ON QT.NRPERFILACESSO = PA.NRPERFILACESSO
                    WHERE QT.IDTIPO = 'D'";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getServicoDescontoMaximoServicoPerfilAcesso(array $params) {

        $sql = "SELECT QT.NRSEQDESCPERF, QT.VRPERCENTUAL,SR.NMSERVORCA,SR.CDSERVORCA, QT.NRPERFILACESSO
                    FROM FCT_PERCDESCPERFIL QT
                    INNER JOIN FCT_SERVICOORCA SR
                    ON SR.CDSERVORCA = QT.CDSERVORCA
                    WHERE QT.IDTIPO = 'D' 
                    AND NRPERFILACESSO = :NRPERFILACESSO";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql,$params);
    }

    public function validaDados(array $params) {

        $sql = "SELECT COUNT(*) AS VALIDA 
                    FROM FCT_PERCDESCPERFIL 
                    WHERE IDTIPO = 'D' 
                    AND NRPERFILACESSO = :NRPERFILACESSO
                    AND CDSERVORCA = :CDSERVORCA
                    AND NRSEQDESCPERF <> :NRSEQDESCPERF";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]['VALIDA'];
    }

    public function insertDescMaxServicoPerfilAcesso (array $params){
        
        $sql = "INSERT INTO FCT_PERCDESCPERFIL (
                NRSEQDESCPERF,        NRPERFILACESSO,   CDSERVORCA,
                VRPERCENTUAL,         NRORG,           IDATIVO,         
                DTINCLUSAO,           NRORGINCLUSAO,   CDOPERINCLUSAO,
                IDTIPO
                )
                VALUES (
                    :NRSEQDESCPERF,        :NRPERFILACESSO,   :CDSERVORCA,
                    :VRPERCENTUAL,         :NRORG,           :IDATIVO,       
                    SYSDATE,               :NRORG,           :CDOPERINCLUSAO,
                    'D'
                )";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateDescMaxServicoPerfilAcesso($params){
        $sql = "UPDATE FCT_PERCDESCPERFIL 
                    SET NRPERFILACESSO  = :NRPERFILACESSO, 
                    CDSERVORCA          = :CDSERVORCA,
                    VRPERCENTUAL        = :VRPERCENTUAL,
                    NRORG               = :NRORG, 
                    IDATIVO             = :IDATIVO,
                    NRORGULTATU         = :NRORG,
                    CDOPERULTATU        = :CDOPERULTATU,
                    DTULTATU            = SYSDATE
                    WHERE NRSEQDESCPERF = :NRSEQDESCPERF";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function deleteDescMaxServicoModalServicoPerfilAcesso($params){
        $sql = "DELETE FROM FCT_PERCDESCPERFIL
                WHERE NRSEQDESCPERF = :NRSEQDESCPERF";

        $this->connection->executeStatement($sql, $params);
    }


}