<?php

namespace SM\Service;

use SM\Factory\Sm00006000TipoDeMedidaFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Sm00006000TipoDeMedidaService {
    protected $connection;
    protected $tipoMedidaFactory;
    protected $environment;
    protected $newCode;

    public function __construct(Sm00006000TipoDeMedidaFactory $tipoMedidaFactory, Environment $environment, NewCode $newCode, EntityManager $entityManager) {
        $this->tipoMedidaFactory            = $tipoMedidaFactory;
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    public function getDescTipoMedida() {
        return $this->tipoMedidaFactory->getDescTipoMedida();
    }

    public function getVariacaoTipoMedida($parametro) {
        return $this->tipoMedidaFactory->getVariacaoTipoMedida($parametro);
    }

    public function saveTabTipoMedida($params) {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            if ($params["CDTPMEDIDASERV"] == NULL) {
                $cdtpmedidaserv = $this->newCode->getCodeByTableName("FCT_TIPOMEDIDASERV", $nrorg, 1, 10, true);
                $parametros = array(
                    "CDTPMEDIDASERV" => $cdtpmedidaserv,
                    "DSTPMEDIDASERV" => $params["DSTPMEDIDASERV"],
                    "IDTPMEDIDASERV" => $params["IDTPMEDIDASERV"],
                    "NRORG"          => $nrorg,
                    "IDATIVO"        => "S",
                    "NRORGINCLUSAO"  => $nrorg,
                    "CDOPERINCLUSAO" => $cdoperador
                );
                $this->tipoMedidaFactory->saveTabTipoMedida($parametros);
            } else {
                $parametros = array(
                    "DSTPMEDIDASERV" => $params["DSTPMEDIDASERV"],
                    "IDTPMEDIDASERV" => $params["IDTPMEDIDASERV"],
                    "NRORG"          => $nrorg,
                    "IDATIVO"        => "S",
                    "CDOPERULTATU"   => $cdoperador,
                    "CDTPMEDIDASERV" => $params["CDTPMEDIDASERV"]
                );
                $this->tipoMedidaFactory->updateTabTipoMedida($parametros);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertVariacaoTipoMedida($params) {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            foreach ($params as $tipomedida) {

                $paramsaux = array(
                    "CDTPMEDIDASERV" => $tipomedida['CDTPMEDIDASERV'],
                    "CDTPMEDIDAVARIAVEL" => $tipomedida['CDTPMEDIDAVARIAVEL'],
                    "NRVARIACAOTPMEDIDA" => $tipomedida['NRVARIACAOTPMEDIDA'],
                    "QTMEDIDAINIVARIACAO" => $tipomedida['QTMEDIDAINIVARIACAO'],
                    "QTMEDIDAFIMVARIACAO" => $tipomedida['QTMEDIDAFIMVARIACAO'],
                );

                $dados = $this->tipoMedidaFactory->validaIntervaloVariacao($paramsaux);

                if ($dados[0]['QTD'] > 0) {
                    throw new \Exception("EXISTE");
                } else {
                    if ($tipomedida["NRVARIACAOTPMEDIDA"] == 0) {
                        $nrvariacaotpmedida = $this->newCode->getCodeByTableName("FCT_VARIACAOTIPOMEDIDA", $nrorg, 1, 10, true);
                        $parametros = array(
                            "NRVARIACAOTPMEDIDA"     => $nrvariacaotpmedida,
                            "CDTPMEDIDASERV"         => $tipomedida['CDTPMEDIDASERV'],
                            "CDTPMEDIDAVARIAVEL"     => $tipomedida['CDTPMEDIDAVARIAVEL'],
                            "QTMEDIDAINIVARIACAO"    => $tipomedida['QTMEDIDAINIVARIACAO'],
                            "QTMEDIDAFIMVARIACAO"    => $tipomedida['QTMEDIDAFIMVARIACAO'],
                            "VRPERCVARIACAO"         => $tipomedida['VRPERCVARIACAO'],
                            "NRORG"                  => $nrorg,
                            "IDATIVO"                => "S",
                            "NRORGINCLUSAO"          => $nrorg,
                            "CDOPERINCLUSAO"         => $cdoperador
                        );

                        $this->tipoMedidaFactory->insertVariacaoTipoMedida($parametros);
                    } else {
                        $parametros = array(
                            "CDTPMEDIDASERV"         => $tipomedida['CDTPMEDIDASERV'],
                            "CDTPMEDIDAVARIAVEL"     => $tipomedida['CDTPMEDIDAVARIAVEL'],
                            "QTMEDIDAINIVARIACAO"    => $tipomedida['QTMEDIDAINIVARIACAO'],
                            "QTMEDIDAFIMVARIACAO"    => $tipomedida['QTMEDIDAFIMVARIACAO'],
                            "VRPERCVARIACAO"         => $tipomedida['VRPERCVARIACAO'],
                            "NRORG"                  => $nrorg,
                            "IDATIVO"                => "S",
                            "NRORGINCLUSAO"          => $nrorg,
                            "CDOPERINCLUSAO"         => $cdoperador,
                            "NRVARIACAOTPMEDIDA"     => $tipomedida['NRVARIACAOTPMEDIDA'],
                        );
                        $this->tipoMedidaFactory->updateVariacaoTipoMedida($parametros);
                    }
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteVariacaoTipoMedida($params) {
        $this->connection->beginTransaction();
        try {
            foreach ($params as $tipomedida) {
                $dataParam = array(
                    "NRVARIACAOTPMEDIDA" => $tipomedida["NRVARIACAOTPMEDIDA"],
                );
                $this->tipoMedidaFactory->deleteVariacaoTipoMedida($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteTipoMedida($params) {
        $this->connection->beginTransaction();
        try {
            foreach ($params as $tipomedida) {
                $dataParam = array(
                    "CDTPMEDIDASERV" => $tipomedida["CDTPMEDIDASERV"],
                );
                $variacao = $this->getVariacaoTipoMedida($dataParam);
                $this->deleteVariacaoTipoMedida($variacao["data"]);
                $this->tipoMedidaFactory->deleteTipoMedida($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
